var path = require('path')
var hljs = require('highlight.js')
var slugify = require('transliteration').slugify;
var striptags = require('./strip-tags');

function resolve(dir) {
  return path.join(__dirname, '..', dir)
}

function convert(str) {
  str = str.replace(/(&#x)(\w{4});/gi, function($0) {
    return String.fromCharCode(parseInt(encodeURIComponent($0).replace(/(%26%23x)(\w{4})(%3B)/g, '$2'), 16));
  });
  return str;
}

var markdown = require('markdown-it')();

module.exports = {
  entry: ['./examples/entry.js', './build/reload.js'],
  output: {
    filename: '[name].[hash:7].js',
    path: resolve("/examples/dist"),
    publicPath: "./", // New
  },
  resolve: {
    extensions: ['.js', '.vue', '.json', '.md'],
    modules: [
      resolve('node_modules')
    ],
    alias: {
      'examples': resolve('examples'),
      'vue$': 'vue/dist/vue.common.js',
      main: resolve('src'),
      lib:resolve('src'),
      'plugins': resolve('examples/plugins/')
    }
  },
  module: {
    loaders: [{
      test: /\.md$/,
      loader: 'vue-markdown-loader',
      options: {
        preset: 'default',
        breaks: true,
        preprocess: function(MarkdownIt, source) {
          MarkdownIt.renderer.rules.table_open = function() {
            return '<table class="table">';
          };
          return source
        },
        use: [
          [require('markdown-it-container'), 'demo', {
            validate: function(params) {
              return params.trim().match(/^demo\s*(.*)$/);
            },

            render: function(tokens, idx) {
              var m = tokens[idx].info.trim().match(/^demo\s*(.*)$/);

              if (tokens[idx].nesting === 1) {
                var description = (m && m.length > 1) ? m[1] : '';
                var content = tokens[idx + 1].content;
                var html = convert(striptags.strip(content, ['script', 'style'])).replace(/(<[^>]*)=""(?=.*>)/g, '$1');

                var script = striptags.fetch(content, 'script');

                var style = striptags.fetch(content, 'style');
                var jsfiddle = { html: html, script: script, style: style };
                var descriptionHTML = description ? markdown.render(description) : '';

                jsfiddle = markdown.utils.escapeHtml(JSON.stringify(jsfiddle));

                return `<demo-block class="demo-box" :jsfiddle="${jsfiddle}">
                    <div class="source" slot="source">${html}</div>
                    ${descriptionHTML}
                    <div class="highlight" slot="highlight">`;
              }

              return '</div></demo-block>\n';
            }
          }]
        ]
      }
    }, { test: /\.vue$/, loader: 'vue-loader' }, {
      test: /\.css$/,
      use: ['style-loader', 'css-loader']
    }, {
      test: /\.(woff2?|eot|ttf|otf|svg)(\?.*)?$/,
      loader: 'url-loader',
      query: {
        limit: 10000,
        name: '[path][name].[ext]'
      }
    }, {
      test: /\.(png|jpe?g|gif|svg)(\?.*)?$/,
      loader: 'url-loader',
      query: {
        limit: 10000,
        name: '[path][name].[hash:7].[ext]'
      }
    }]
  }
}
