'use strict';

exports.__esModule = true;
exports.onRoamingMsgs = onRoamingMsgs;
exports.onOfflineMsgs = onOfflineMsgs;
exports.onMsg = onMsg;
exports.sendMsg = sendMsg;
exports.sendFileMsg = sendFileMsg;
exports.sendGeo = sendGeo;
exports.sendMsgReceipt = sendMsgReceipt;
exports.getHistoryMsgs = getHistoryMsgs;
exports.resetNoMoreHistoryMsgs = resetNoMoreHistoryMsgs;

var _utils = require('../../../utils');

var _utils2 = _interopRequireDefault(_utils);

var _configs = require('../../../configs');

var _configs2 = _interopRequireDefault(_configs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function formatMsg(state, msg) {
  var nim = state.nim;
  if (msg.type === 'robot') {
    if (msg.content && msg.content.flag === 'bot') {
      if (msg.content.message) {
        msg.content.message = msg.content.message.map(function (item) {
          switch (item.type) {
            case 'template':
              item.content = nim.parseRobotTemplate(item.content);
              break;
            case 'text':
            case 'image':
            case 'answer':
              break;
          }
          return item;
        });
      }
    }
  }
  return msg;
}

function onRoamingMsgs(_ref, obj) {
  var state = _ref.state,
      commit = _ref.commit;

  // commit('updateMsgs', obj.msgs)
  var msgs = obj.msgs.map(function (msg) {
    return formatMsg(state, msg);
  });
  // console.log("onRoamingMsgs:::", msgs)
  commit('updateMsgs', msgs);
}

function onOfflineMsgs(_ref2, obj) {
  var state = _ref2.state,
      commit = _ref2.commit;


  var msgs = obj.msgs.map(function (msg) {
    return formatMsg(state, msg);
  });
  commit('updateMsgs', msgs);
}

function onMsg(_ref3, msg) {
  var state = _ref3.state,
      commit = _ref3.commit,
      dispatch = _ref3.dispatch;

  // if (msg.flow === "out") {
  //   msg.sessionId = "p2p-" + msg.target
  // } else if (msg.flow === "in") {
  //   msg.sessionId = "p2p-" + msg.from
  // }
  commit('putMsg', msg);

  if (msg.sessionId === state.currSessionId) {
    commit('updateCurrSessionMsgs', {
      type: 'put',
      msg: msg
    });
    // 发送已读回执
    dispatch('sendMsgReceipt');
  }
}

//消息发送回调
function onSendMsgDone(_ref4, error, msg) {
  var state = _ref4.state,
      commit = _ref4.commit,
      dispatch = _ref4.dispatch;

  // console.log("onSendMsgDone::::", JSON.stringify(msg))
  if (error) {
    // 被拉黑
    if (error.code === 7101) {
      msg.status = 'defriend';
      // console.log("onSendMsgDone:::", error.message)
      // console.log("被拉黑了...")
    } else {
        // console.log("onSendMsgDone:::", error.message)
      }
  }
  dispatch('onMsg', msg);
  // onMsg({ state, commit, dispatch }, msg)
}

// 发送普通消息
function sendMsg(_ref5, obj) {
  var state = _ref5.state,
      commit = _ref5.commit,
      dispatch = _ref5.dispatch;

  var nim = state.nim;
  obj = obj || {};
  var type = obj.type || '';
  // console.log("sendMsg:::::::", obj.to)
  switch (type) {
    case 'text':
      nim.sendText({
        scene: obj.scene,
        to: obj.to,
        text: obj.text,
        done: function done(error, msg) {
          onSendMsgDone({ state: state, commit: commit, dispatch: dispatch }, error, msg);
        }
      });
      break;
    case 'custom':
      nim.sendCustomMsg({
        scene: obj.scene,
        to: obj.to,
        pushContent: obj.pushContent,
        content: JSON.stringify(obj.content),
        done: function done(error, msg) {
          onSendMsgDone({ state: state, commit: commit, dispatch: dispatch }, error, msg);
        }
      });
  }
}

// 发送文件消息
function sendFileMsg(_ref6, obj) {
  var state = _ref6.state,
      commit = _ref6.commit,
      dispatch = _ref6.dispatch;

  var nim = state.nim;
  var scene = obj.scene,
      to = obj.to,
      fileInput = obj.fileInput,
      custom = obj.custom,
      pushContent = obj.pushContent;

  var type = 'file';
  if (/\.(png|jpg|bmp|jpeg|gif)$/i.test(fileInput.value)) {
    type = 'image';
  } else if (/\.(mov|mp4|ogg|webm)$/i.test(fileInput.value)) {
    type = 'video';
  }
  custom = JSON.stringify(custom);
  nim.sendFile({
    scene: scene,
    to: to,
    type: type,
    fileInput: fileInput,
    custom: custom,
    pushContent: pushContent,
    uploadprogress: function uploadprogress(data) {
      // // console.log(data.percentageText)
    },
    uploaderror: function uploaderror() {
      // console && // console.log('上传失败')
    },
    uploaddone: function uploaddone(error, file) {
      // // console.log(error);
      // // console.log(file);
    },
    beforesend: function beforesend(msg) {
      // console && // console.log('正在发送消息, id=', msg);
    },
    done: function done(error, msg) {
      onSendMsgDone({ state: state, commit: commit, dispatch: dispatch }, error, msg);
    }
  });
}

function sendGeo(_ref7, obj) {
  var state = _ref7.state,
      commit = _ref7.commit,
      dispatch = _ref7.dispatch;

  var nim = state.nim;
  var scene = obj.scene,
      to = obj.to,
      geo = obj.geo,
      custom = obj.custom,
      pushContent = obj.pushContent;

  custom = JSON.stringify(custom);
  nim.sendGeo({
    scene: scene,
    to: to,
    geo: geo,
    custom: custom,
    pushContent: pushContent,
    done: function done(error, msg) {
      onSendMsgDone({ state: state, commit: commit, dispatch: dispatch }, error, msg);
    }
  });
}

// 发送消息已读回执
function sendMsgReceipt(_ref8) {
  var state = _ref8.state,
      commit = _ref8.commit;

  // 如果有当前会话
  var currSessionId = state.currSessionId;
  if (currSessionId) {
    // 只有点对点消息才发已读回执
    if (_utils2.default.parseSession(currSessionId).scene === 'p2p') {
      var msgs = state.currSessionMsgs;
      var nim = state.nim;

      if (state.sessionMap[currSessionId]) {
        nim.sendMsgReceipt({
          msg: state.sessionMap[currSessionId].lastMsg,
          done: function sendMsgReceiptDone(error, obj) {
            // do something
            // console.log('发送消息已读回执' + (!error ? '成功' : '失败'), error, obj);
          }
        });
      }
    }
  }
}

function sendMsgReceiptDone(error, obj) {
  // console.log('发送消息已读回执' + (!error ? '成功' : '失败'), error, obj);
}

function getHistoryMsgs(_ref9, obj) {
  var state = _ref9.state,
      commit = _ref9.commit;

  var nim = state.nim;
  if (nim) {
    var scene = obj.scene,
        to = obj.to;

    var options = {
      scene: scene,
      to: to,
      reverse: false,
      asc: true,
      limit: _configs2.default.localMsglimit || 20,
      done: function getHistoryMsgsDone(error, obj) {
        if (obj.msgs) {
          if (obj.msgs.length === 0) {
            commit('setNoMoreHistoryMsgs');
          } else {
            commit('updateCurrSessionMsgs', {
              type: 'concat',
              msgs: obj.msgs
            });
          }
        }
      }
    };
    if (state.currSessionLastMsg) {
      options = Object.assign(options, {
        lastMsgId: state.currSessionLastMsg.idServer,
        endTime: state.currSessionLastMsg.time
      });
    }
    nim.getHistoryMsgs(options);
  }
}

function resetNoMoreHistoryMsgs(_ref10) {
  var commit = _ref10.commit;

  commit('resetNoMoreHistoryMsgs');
}