'use strict';

exports.__esModule = true;

var _cookie = require('../../../utils/cookie.js');

var _cookie2 = _interopRequireDefault(_cookie);

var _configs = require('../../../configs');

var _configs2 = _interopRequireDefault(_configs);

var _utils = require('../../../utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {
  updateUserUID: function updateUserUID(state, loginInfo) {
    // console.log("updateUserUID:::", loginInfo)
    state.userUID = loginInfo.uid;
    state.sdktoken = loginInfo.sdktoken;
    _cookie2.default.setCookie('uid', loginInfo.uid);
    _cookie2.default.setCookie('sdktoken', loginInfo.sdktoken);
  },
  updateCurrChatroomMsgs: function updateCurrChatroomMsgs(state, obj) {
    var _Object$assign = Object.assign({}, obj),
        type = _Object$assign.type,
        msgs = _Object$assign.msgs;

    if (type === 'put') {
      msgs.forEach(function (msg) {
        var chatroomId = msg.chatroomId;
        if (chatroomId === state.currChatroomId) {
          msgs.forEach(function (msg) {
            state.currChatroomMsgs.push(msg);
          });
        }
      });
    } else if (type === 'concat') {
      // 一般用于历史消息拼接
      var currSessionMsgs = obj.msgs;
      currSessionMsgs.reverse();
      currSessionMsgs.forEach(function (msg) {
        state.currSessionMsgs.unshift(msg);
      });
      if (obj.msgs[0]) {
        state.currSessionLastMsg = obj.msgs[0];
      }
    }
  },

  // 初始化，收到离线漫游消息时调用
  updateMsgs: function updateMsgs(state, msgs) {
    // console.log("updateMsgs:::")
    var nim = state.nim;
    var tempSessionMap = {};
    msgs.forEach(function (msg) {
      var sessionId = msg.sessionId;
      tempSessionMap[sessionId] = true;
      if (!state.msgs[sessionId]) {
        state.msgs[sessionId] = [];
      }
      // sdk会做消息去重
      state.msgs[sessionId] = nim.mergeMsgs(state.msgs[sessionId], [msg]);
      // state.msgs[sessionId].push(msg)
    });

    store.commit('updateMsgByIdClient', msgs);
    for (var sessionId in tempSessionMap) {
      state.msgs[sessionId].sort(function (a, b) {
        return a.time - b.time;
      });

      if (sessionId === state.currSessionId) {
        store.commit('updateCurrSessionMsgs', {
          type: 'init'
        });
      }
    }
  },

  // 更新追加消息，追加一条消息
  putMsg: function putMsg(state, msg) {
    var sessionId = msg.sessionId;
    if (!state.msgs[sessionId]) {
      state.msgs[sessionId] = [];
    }
    store.commit('updateMsgByIdClient', msg);
    var tempMsgs = state.msgs[sessionId];
    var lastMsgIndex = tempMsgs.length - 1;
    if (tempMsgs.length === 0 || msg.time >= tempMsgs[lastMsgIndex].time) {
      tempMsgs.push(msg);
    } else {
      for (var i = lastMsgIndex; i >= 0; i--) {
        var currMsg = tempMsgs[i];
        if (msg.time >= currMsg.time) {
          state.msgs[sessionId].splice(i, 0, msg);
          break;
        }
      }
    }
  },

  // 用idClient 更新消息，目前用于消息撤回
  updateMsgByIdClient: function updateMsgByIdClient(state, msgs) {
    if (!Array.isArray(msgs)) {
      msgs = [msgs];
    }
    var tempTime = new Date().getTime();
    msgs.forEach(function (msg) {
      // 有idClient 且 5分钟以内的消息
      if (msg.idClient && tempTime - msg.time < 1000 * 300) {
        state.msgsMap[msg.idClient] = msg;
      }
    });
  },

  // 更新当前会话id，用于唯一判定是否在current session状态
  updateCurrSessionId: function updateCurrSessionId(state, obj) {
    var type = obj.type || '';
    if (type === 'destroy') {
      state.currSessionId = null;
    } else if (type === 'init') {
      if (obj.sessionId && obj.sessionId !== state.currSessionId) {
        state.currSessionId = obj.sessionId;
      }
    }
  },

  // 更新当前会话列表的聊天记录，包括历史消息、单聊消息等，不包括聊天室消息
  // replace: 替换idClient的消息
  updateCurrSessionMsgs: function updateCurrSessionMsgs(state, obj) {
    var type = obj.type || '';
    // console.log('type的值是'+type)
    if (type === 'destroy') {
      //清空会话消息
      // console.log('type的值是destroy,退出聊天应该走这里')
      state.currSessionMsgs = [];
      state.currSessionLastMsg = null;
      store.commit('updateCurrSessionId', {
        type: 'destroy'
      });
    } else if (type === 'init') {
      // 初始化会话消息列表
      if (state.currSessionId) {
        var sessionId = state.currSessionId;
        var currSessionMsgs = [].concat(state.msgs[sessionId] || []);
        // 做消息截断
        var limit = _configs2.default.localMsglimit;
        var msgLen = currSessionMsgs.length;
        if (msgLen - limit > 0) {
          state.currSessionLastMsg = currSessionMsgs[msgLen - limit];
          currSessionMsgs = currSessionMsgs.slice(msgLen - limit, msgLen);
        } else if (msgLen > 0) {
          state.currSessionLastMsg = currSessionMsgs[0];
        } else {
          state.currSessionLastMsg = null;
        }
        state.currSessionMsgs = [];
        var lastMsgTime = 0;

        currSessionMsgs.forEach(function (msg) {
          if (msg.time - lastMsgTime > 1000 * 60 * 5) {
            lastMsgTime = msg.time;
            state.currSessionMsgs.push({
              type: 'timeTag',
              text: _utils2.default.formatDate(msg.time, false)
            });
          }
          state.currSessionMsgs.push(msg);
        });
      }
    } else if (type === 'put') {
      // 追加一条消息

      // console.log("开始追加信息...")
      var newMsg = obj.msg;
      var _lastMsgTime = 0;
      var lenCurrMsgs = state.currSessionMsgs.length;
      if (lenCurrMsgs > 0) {
        _lastMsgTime = state.currSessionMsgs[lenCurrMsgs - 1].time;
      }
      // console.log("开始追加信息...222...")
      if (newMsg) {
        if (newMsg.time - _lastMsgTime > 1000 * 60 * 5) {
          state.currSessionMsgs.push({
            type: 'timeTag',
            text: _utils2.default.formatDate(newMsg.time, false)
          });
        }
        state.currSessionMsgs.push(newMsg);
        // console.log("开始追加信息...state.currSessionMsgs...", state.currSessionMsgs)
      }
    } else if (type === 'concat') {
      // 一般用于历史消息拼接
      var _currSessionMsgs = [];
      var _lastMsgTime2 = 0;
      obj.msgs.forEach(function (msg) {
        if (msg.time - _lastMsgTime2 > 1000 * 60 * 5) {
          _lastMsgTime2 = msg.time;
          _currSessionMsgs.push({
            type: 'timeTag',
            text: _utils2.default.formatDate(msg.time, false)
          });
        }
        _currSessionMsgs.push(msg);
      });
      _currSessionMsgs.reverse();
      _currSessionMsgs.forEach(function (msg) {
        state.currSessionMsgs.unshift(msg);
      });
      if (obj.msgs[0]) {
        state.currSessionLastMsg = obj.msgs[0];
      }
    } else if (type === 'replace') {
      var _msgLen = state.currSessionMsgs.length;
      var lastMsgIndex = _msgLen - 1;
      if (_msgLen > 0) {
        for (var i = lastMsgIndex; i >= 0; i--) {
          if (state.currSessionMsgs[i].idClient === obj.idClient) {
            state.currSessionMsgs.splice(i, 1, obj.msg);
            break;
          }
        }
      }
    }
  },
  updateSessions: function updateSessions(state, sessions) {
    var nim = state.nim;

    state.sessionlist = nim.mergeSessions(state.sessionlist, sessions);
    state.sessionlist = state.sessionlist.filter(function (item) {
      if (/^team-/.test(item.id)) {
        return false;
      }
      return true;
    });

    state.sessionlist.sort(function (a, b) {
      return b.updateTime - a.updateTime;
    });
    state.sessionlist.forEach(function (item) {
      state.sessionMap[item.id] = item;
    });
  },
  setNoMoreHistoryMsgs: function setNoMoreHistoryMsgs(state) {
    state.noMoreHistoryMsgs = true;
  },
  resetNoMoreHistoryMsgs: function resetNoMoreHistoryMsgs(state) {
    state.noMoreHistoryMsgs = false;
  },
  updateTargetInfo: function updateTargetInfo(state, obj) {
    state.target = obj;
  },
  updateIMUserInfo: function updateIMUserInfo(state, obj) {
    state.imUserInfo = obj;
    // cookie.setCookie('uid', obj.uid)
    // cookie.setCookie('sdktoken', obj.sdktoken)
  },
  updateNIM: function updateNIM(state, nimObj) {
    state.nim = nimObj;
  },
  updateEnvironment: function updateEnvironment(state, env) {
    state.environment = env;
  },

  //更新系统消息
  updateCustomSysMsgs: function updateCustomSysMsgs(state, sysMsgs) {
    if (!Array.isArray(sysMsgs)) {
      sysMsgs = [sysMsgs];
    }
    sysMsgs = sysMsgs.map(function (msg) {
      msg.showTime = _utils2.default.formatDate(msg.time, false);
      return msg;
    });
    state.customSysMsgs = state.customSysMsgs.concat(sysMsgs);
  }
};