export default {
  init(opt){
    this.paint = false
    this.width = opt.width
    this.height = opt.height
    this.selector = opt.selector
    this.img = opt.img

    let cvs = document.createElement('canvas')
    cvs.width = this.width
    cvs.height = this.height

    let self = this

    this.outlineImage = new Image()
    this.outlineImage.setAttribute('crossOrigin', 'anonymous') 
    this.outlineImage.onload = function() {
      self.redraw()
    }

    if(!!this.img.src && (this.img.src.indexOf('http://') > -1 || this.img.src.indexOf('https://') > -1)){
      this.outlineImage.src = this.hasTimestampHandle(this.img.src)+'?timestamp='+new Date().getTime()
    } else {
      this.outlineImage.src = this.img.src
    }

    this.selector.appendChild(cvs)

    this.context = cvs.getContext('2d')

    let canvas = this.selector.querySelectorAll('canvas')[0]
		
    canvas.addEventListener('touchstart',function(e){
      let pageX = e.touches[0].pageX,
        pageY = e.touches[0].pageY
      var mouseX = pageX - canvas.getBoundingClientRect().left
      var mouseY = pageY - canvas.getBoundingClientRect().top
      // if(navigator.userAgent.indexOf('Android') > -1 || navigator.userAgent.indexOf('Adr') > -1){
      // 	mouseY = pageY - this.offsetTop + (window.screen.height - window.innerHeight)/2 + (18/570)*document.body.clientHeight;
      // } else {
      // 	mouseY = pageY - this.offsetTop + (window.screen.height - window.innerHeight)/2;
      // }

      self.paint = true
      self.addClick(mouseX, mouseY, false)
      self.redraw()
      e.preventDefault()
    },false)

    canvas.addEventListener('touchmove',function(e){
      if(self.paint==true){
        let pageX = e.touches[0].pageX,
          pageY = e.touches[0].pageY

        var mouseX = pageX - canvas.getBoundingClientRect().left
        var mouseY = pageY - canvas.getBoundingClientRect().top
        // if(navigator.userAgent.indexOf('Android') > -1 || navigator.userAgent.indexOf('Adr') > -1){
        // 	mouseY = pageY - this.offsetTop + (window.screen.height - window.innerHeight)/2 + (18/570)*document.body.clientHeight;
        // } else {
        // 	mouseY = pageY - this.offsetTop + (window.screen.height - window.innerHeight)/2;
        // }

        self.addClick(mouseX, mouseY, true)
        self.redraw()
      }
      e.preventDefault()
    },false)

    canvas.addEventListener('touchend',function(e){
      self.paint = false
      e.preventDefault()
    },false)

    this.clickX = new Array()
    this.clickY = new Array()
    this.clickTool = new Array()
    this.clickColor = new Array()
    this.clickSize = new Array()
    this.clickDrag = new Array()
    this.paint = false
    this.canvasWidth = this.width
    this.canvasHeight = this.height

    this.curSize = opt.initSize
    this.curColor = opt.initColor
    this.curTool = opt.initTool
    // this.curTool = 'eraser'
  },
  toDataURL(){
    this.selector.querySelectorAll('canvas')[0].setAttribute('crossOrigin', 'anonymous')
    return this.selector.querySelectorAll('canvas')[0].toDataURL()
  },
  hasTimestampHandle(url){
    var index = url.indexOf('timestamp')
    if(index>-1){
      return url.substring(0,index-1)
    }else{
      return url
    }
  },
  destory(){
    this.selector.querySelectorAll('canvas')[0].remove()
  },
  addClick(x, y, dragging){
    this.clickX.push(x)
    this.clickY.push(y)
    this.clickTool.push(this.curTool)
    this.clickColor.push(this.curColor)
    this.clickSize.push(this.curSize)
    this.clickDrag.push(dragging)
  },
  redraw(){
    this.clearCanvas()
    let canvas = document.createElement('canvas')
    canvas.width = this.canvasWidth
    canvas.height = this.canvasHeight
    let context = canvas.getContext('2d')

    var radius
    var i = 0
    for(; i < this.clickX.length; i++)
    {		
      if(this.clickSize[i] == 'small'){
        radius = 2
      }else if(this.clickSize[i] == 'normal'){
        radius = 5
      }else if(this.clickSize[i] == 'large'){
        radius = 10
      }else if(this.clickSize[i] == 'huge'){
        radius = 20
      }else{
        alert('Error: Radius is zero for click ' + i)
        radius = 0	
      }
			
      context.beginPath()
			
      if(this.clickDrag[i] && i){
        context.moveTo(this.clickX[i-1], this.clickY[i-1])
      }else{
        context.moveTo(this.clickX[i], this.clickY[i])
      }
      context.lineTo(this.clickX[i], this.clickY[i])
      context.closePath()
			
      if(this.clickTool[i] == 'eraser'){
        context.globalCompositeOperation = 'destination-out'
        context.strokeStyle = 'white'
      }else{
        context.globalCompositeOperation = 'source-over'
        context.strokeStyle = this.clickColor[i]
      }
      context.lineJoin = 'round'
      context.lineWidth = radius
      context.stroke()
			
    }
    context.restore()

    // Draw the outline image
    this.context.drawImage(this.outlineImage, 0, 0, this.canvasWidth, this.canvasHeight)
    this.context.drawImage(canvas, 0, 0, this.canvasWidth, this.canvasHeight)
  },
  clearCanvas(){
    this.context.clearRect(0, 0, this.canvasWidth, this.canvasHeight)
  },
  setTool(val){
    this.curTool = val
  },
  setColor(val){
    this.curColor = val
  },
  setSize(val){
    this.curSize = val
  }
}