/* 
 * SDK连接相关
 */

import configs from '../../../configs'

import { onSessions, onUpdateSession } from './session'

import { onRoamingMsgs, onOfflineMsgs, onMsg } from './msgs'

import { onCustomSysMsgs } from './sysMsgs'

import util from '../../../utils'

export function initNimSDK({ state, commit, dispatch }, loginInfo) {
  // console.log("in initNimSDK")
  if (state.nim) {
    state.nim.disconnect()
  }
  // console.log("configs.appkey[state.environment]:::", configs.appkey[state.environment])
  // console.log("loginInfo.uid:::", loginInfo.uid)
  // console.log("loginInfo.sdktoken:::", loginInfo.sdktoken)
  // 初始化SDK
  window.nim = state.nim = SDK.NIM.getInstance({
    // debug: true && { api: 'info', style: 'font-size:12px;color:blue;background-color:rgba(0,0,0,0.1)' },
    appKey: configs.appkey[state.environment],
    account: loginInfo.uid,
    token: loginInfo.sdktoken,
    db: false,
    syncSessionUnread: true,
    syncRobots: true,
    autoMarkRead: true, // 默认为true
    syncRoamingMsgs: true,
    onconnect: function onConnect(event) {
      if (loginInfo) {
        // 连接上以后更新uid
        commit('updateUserUID', loginInfo)
      }
    },
    onerror: function onError(event) {
      console.error('网络连接状态异常')
    },
    onwillreconnect: function onWillReconnect() {
      // console.log(event)
    },
    ondisconnect: function onDisconnect(error) {
      // console.log("ondisconnect::::", error)
      switch (error.code) {
        // 账号或者密码错误, 请跳转到登录页面并提示错误
        case 302:
          // pageUtil.turnPage('帐号或密码错误', 'login')
          break;
          // 被踢, 请提示错误后跳转到登录页面
        case 'kicked':
          let map = {
            PC: '电脑版',
            Web: '网页版',
            Android: '手机版',
            iOS: '手机版',
            WindowsPhone: '手机版'
          }
          let str = error.from
          let errorMsg = `你的帐号于${util.formatDate(new Date())}被${(map[str]||'其他端')}踢出下线，请确定帐号信息安全!`
          console.error(errorMsg)
          break
        default:
          break
      }
    },
    // // 多端登录
    // onloginportschange: onLoginPortsChange,

    // // 会话
    onsessions: function(obj) {
      onSessions({ state, commit, dispatch }, obj)
    },
    onupdatesession: function(obj) {
      onUpdateSession({ state, commit, dispatch }, obj)
    },
    // // 消息
    onroamingmsgs: function(obj) {
      onRoamingMsgs({ state, commit, dispatch }, obj)
    },
    onofflinemsgs: function(obj) {
      onOfflineMsgs({ state, commit, dispatch }, obj)
    },
    onmsg: function(msg) {
      onMsg({ state, commit, dispatch }, msg)
    },
    oncustomsysmsg: function(obj) {
      onCustomSysMsgs({ state, commit, dispatch }, obj)
    },
    // 同步完成
    onsyncdone: function onSyncDone() {
      // dispatch('hideLoading')
      // 说明在聊天列表页
      if (state.currSessionId) {
        dispatch('setCurrSession', state.currSessionId)
      }
    }
  })
  // commit('updateNIM',window.nim)
}
