import util from '../../../utils'
import config from '../../../configs'


function formatMsg(state, msg) {
  const nim = state.nim
  if (msg.type === 'robot') {
    if (msg.content && msg.content.flag === 'bot') {
      if (msg.content.message) {
        msg.content.message = msg.content.message.map(item => {
          switch (item.type) {
          case 'template':
            item.content = nim.parseRobotTemplate(item.content)
            break
          case 'text':
          case 'image':
          case 'answer':
            break
          }
          return item
        })
      }
    }
  }
  return msg
}

export function onRoamingMsgs({ state, commit }, obj) {
  // commit('updateMsgs', obj.msgs)
  let msgs = obj.msgs.map(msg => {
    return formatMsg(state, msg)
  })
  // console.log("onRoamingMsgs:::", msgs)
  commit('updateMsgs', msgs)
}

export function onOfflineMsgs({ state, commit }, obj) {

  let msgs = obj.msgs.map(msg => {
    return formatMsg(state, msg)
  })
  commit('updateMsgs', msgs)
}

export function onMsg({ state, commit, dispatch }, msg) {
  // if (msg.flow === "out") {
  //   msg.sessionId = "p2p-" + msg.target
  // } else if (msg.flow === "in") {
  //   msg.sessionId = "p2p-" + msg.from
  // }
  commit('putMsg', msg)

  if (msg.sessionId === state.currSessionId) {
    commit('updateCurrSessionMsgs', {
      type: 'put',
      msg
    })
    // 发送已读回执
    dispatch('sendMsgReceipt')
  }
}

//消息发送回调
function onSendMsgDone({ state, commit, dispatch }, error, msg) {
  // console.log("onSendMsgDone::::", JSON.stringify(msg))
  if (error) {
    // 被拉黑
    if (error.code === 7101) {
      msg.status = 'defriend'
      // console.log("onSendMsgDone:::", error.message)
      // console.log("被拉黑了...")
    } else {
      // console.log("onSendMsgDone:::", error.message)
    }
  }
  dispatch('onMsg', msg)
  // onMsg({ state, commit, dispatch }, msg)
}

// 发送普通消息
export function sendMsg({ state, commit, dispatch }, obj) {
  const nim = state.nim
  obj = obj || {}
  let type = obj.type || ''
  // console.log("sendMsg:::::::", obj.to)
  switch (type) {
  case 'text':
    nim.sendText({
      scene: obj.scene,
      to: obj.to,
      text: obj.text,
      done: function(error, msg) {
        onSendMsgDone({ state, commit, dispatch }, error, msg)
      }
    })
    break
  case 'custom':
    nim.sendCustomMsg({
      scene: obj.scene,
      to: obj.to,
      pushContent: obj.pushContent,
      content: JSON.stringify(obj.content),
      done: function(error, msg) {
        onSendMsgDone({ state, commit, dispatch }, error, msg)
      }
    })
  }
}

// 发送文件消息
export function sendFileMsg({ state, commit, dispatch }, obj) {
  const nim = state.nim
  let { scene, to, fileInput, custom, pushContent } = obj
  let type = 'file'
  if (/\.(png|jpg|bmp|jpeg|gif)$/i.test(fileInput.value)) {
    type = 'image'
  } else if (/\.(mov|mp4|ogg|webm)$/i.test(fileInput.value)) {
    type = 'video'
  }
  custom = JSON.stringify(custom)
  nim.sendFile({
    scene,
    to,
    type,
    fileInput,
    custom,
    pushContent,
    uploadprogress: function(data) {
      // // console.log(data.percentageText)
    },
    uploaderror: function() {
      // console && // console.log('上传失败')
    },
    uploaddone: function(error, file) {
      // // console.log(error);
      // // console.log(file);
    },
    beforesend: function(msg) {
      // console && // console.log('正在发送消息, id=', msg);
    },
    done: function(error, msg) {
      onSendMsgDone({ state, commit, dispatch }, error, msg)
    }
  })
}

export function sendGeo({ state, commit, dispatch }, obj) {
  const nim = state.nim
  let { scene, to, geo, custom, pushContent } = obj
  custom = JSON.stringify(custom)
  nim.sendGeo({
    scene,
    to,
    geo,
    custom,
    pushContent,
    done: function(error, msg) {
      onSendMsgDone({ state, commit, dispatch }, error, msg)
    }
  })
}

// 发送消息已读回执
export function sendMsgReceipt({ state, commit }) {
  // 如果有当前会话
  let currSessionId = state.currSessionId
  if (currSessionId) {
    // 只有点对点消息才发已读回执
    if (util.parseSession(currSessionId).scene === 'p2p') {
      let msgs = state.currSessionMsgs
      const nim = state.nim

      if (state.sessionMap[currSessionId]) {
        nim.sendMsgReceipt({
          msg: state.sessionMap[currSessionId].lastMsg,
          done: function sendMsgReceiptDone(error, obj) {
            // do something
            // console.log('发送消息已读回执' + (!error ? '成功' : '失败'), error, obj);
          }
        })
      }
    }
  }
}

function sendMsgReceiptDone(error, obj) {
  // console.log('发送消息已读回执' + (!error ? '成功' : '失败'), error, obj);
}

export function getHistoryMsgs({ state, commit }, obj) {
  const nim = state.nim
  if (nim) {
    let { scene, to } = obj
    let options = {
      scene,
      to,
      reverse: false,
      asc: true,
      limit: config.localMsglimit || 20,
      done: function getHistoryMsgsDone(error, obj) {
        if (obj.msgs) {
          if (obj.msgs.length === 0) {
            commit('setNoMoreHistoryMsgs')
          } else {
            commit('updateCurrSessionMsgs', {
              type: 'concat',
              msgs: obj.msgs
            })
          }
        }
      }
    }
    if (state.currSessionLastMsg) {
      options = Object.assign(options, {
        lastMsgId: state.currSessionLastMsg.idServer,
        endTime: state.currSessionLastMsg.time,
      })
    }
    nim.getHistoryMsgs(options)
  }
}

export function resetNoMoreHistoryMsgs({ commit }) {
  commit('resetNoMoreHistoryMsgs')
}
